@@ -1,8 +1,8 @@
 # BEGIN BPS TAGGED BLOCK {{{
 # 
 # COPYRIGHT:
-#  
-# This software is Copyright (c) 1996-2009 Best Practical Solutions, LLC 
+# 
+# This software is Copyright (c) 1996-2009 Best Practical Solutions, LLC
 #                                          <jesse@bestpractical.com>
 # 
 # (Except where explicitly superseded by other copyright notices)
@@ -45,6 +45,7 @@
 # those contributions and any derivatives thereof.
 # 
 # END BPS TAGGED BLOCK }}}
+
 =head1 NAME 
 
 RT::System
@@ -65,15 +66,16 @@ In the future, there will probably be other API goodness encapsulated here.
 
 
 package RT::System;
-use base qw /RT::Record/;
+
 use strict;
+use warnings;
+use base qw/RT::Record/;
 
 use RT::ACL;
-use vars qw/ $RIGHTS/;
 
 # System rights are rights granted to the whole system
 # XXX TODO Can't localize these outside of having an object around.
-$RIGHTS = {
+our $RIGHTS = {
     SuperUser              => 'Do anything and everything',           # loc_pair
     AdminAllPersonalGroups =>
       "Create, delete and modify the members of any user's personal groups"
@@ -85,6 +87,7 @@ $RIGHTS = {
     DelegateRights =>
       "Delegate specific rights which have been granted to you.",     # loc_pair
     ShowConfigTab => "show Configuration tab",     # loc_pair
+    ShowApprovalsTab => "show Approvals tab",     # loc_pair
     LoadSavedSearch => "allow loading of saved searches",     # loc_pair
     CreateSavedSearch => "allow creation of saved searches",      # loc_pair
 };
@@ -99,25 +102,20 @@ foreach my $right ( keys %{$RIGHTS} ) {
 
 =head2 AvailableRights
 
-Returns a hash of available rights for this object. The keys are the right names and the values are a description of what the rights do
-
-=begin testing
-
-my $s = RT::System->new($RT::SystemUser);
-my $rights = $s->AvailableRights;
-ok ($rights, "Rights defined");
-ok ($rights->{'AdminUsers'},"AdminUsers right found");
-ok ($rights->{'CreateTicket'},"CreateTicket right found");
-ok ($rights->{'AdminGroupMembership'},"ModifyGroupMembers right found");
-ok (!$rights->{'CasdasdsreateTicket'},"bogus right not found");
-
-
-
-=end testing
+Returns a hash of available rights for this object.
+The keys are the right names and the values are a
+description of what the rights do.
 
+This method as well returns rights of other RT objects,
+like L<RT::Queue> or L<RT::Group>. To allow users to apply
+those rights globally.
 
 =cut
 
+
+use RT::CustomField;
+use RT::Queue;
+use RT::Group; 
 sub AvailableRights {
     my $self = shift;
 
@@ -125,15 +123,31 @@ sub AvailableRights {
     my $group = RT::Group->new($RT::SystemUser);
     my $cf    = RT::CustomField->new($RT::SystemUser);
 
-    my $qr =$queue->AvailableRights();
+    my $qr = $queue->AvailableRights();
     my $gr = $group->AvailableRights();
     my $cr = $cf->AvailableRights();
 
     # Build a merged list of all system wide rights, queue rights and group rights.
     my %rights = (%{$RIGHTS}, %{$gr}, %{$qr}, %{$cr});
+
     return(\%rights);
 }
 
+=head2 AddRights C<RIGHT>, C<DESCRIPTION> [, ...]
+
+Adds the given rights to the list of possible rights.  This method
+should be called during server startup, not at runtime.
+
+=cut
+
+sub AddRights {
+    my $self = shift if ref $_[0] or $_[0] eq __PACKAGE__;
+    my %new = @_;
+    $RIGHTS = { %$RIGHTS, %new };
+    %RT::ACE::LOWERCASERIGHTNAMES = ( %RT::ACE::LOWERCASERIGHTNAMES,
+                                      map { lc($_) => $_ } keys %new);
+}
+
 sub _Init {
     my $self = shift;
     $self->SUPER::_Init (@_) if @_ && $_[0];
@@ -143,24 +157,10 @@ sub _Init {
 
 Returns RT::System's id. It's 1. 
 
-
-=begin testing
-
-use RT::System;
-my $sys = RT::System->new();
-is( $sys->Id, 1);
-is ($sys->id, 1);
-
-=end testing
-
-
 =cut
 
 *Id = \&id;
-
-sub id {
-    return (1);
-}
+sub id { return 1 }
 
 =head2 Load
 
@@ -169,18 +169,26 @@ It does nothing
 
 =cut
 
-sub Load {
-	return (1);
-}
+sub Load    { return 1 }
+sub Name    { return 'RT System' }
+sub __Set   { return 0 }
+sub __Value { return 0 }
+sub Create  { return 0 }
+sub Delete  { return 0 }
 
-sub Name {
-    return 'RT System';
-}
+sub SubjectTag {
+    my $self = shift;
+    my $queue = shift;
 
-sub __Set { 0 }
-sub __Value { 0 }
-sub Create { 0 }
-sub Delete { 0 }
+    my $map = $self->FirstAttribute('BrandedSubjectTag');
+    $map = $map->Content if $map;
+    return $queue ? undef : () unless $map;
+
+    return $map->{ $queue->id } if $queue;
+
+    my %seen = ();
+    return grep !$seen{lc $_}++, values %$map;
+}
 
 eval "require RT::System_Vendor";
 die $@ if ($@ && $@ !~ qr{^Can't locate RT/System_Vendor.pm});
